/**
* \file global.h
* \brief In dieser Datei sind Variablen, Funktionen und Defines die im ganzen Programm zur verfuegugn stehen muessen
* \author Philipp Kaelin 
*/

#ifndef GLOBAL_H
	#define GLOBAL_H


	//***********
	//* Includes
	//***********
	#include <avr/pgmspace.h>
	

	//**********
	//* Defines
	//**********
	//#define DEBUG					TRUE

	// Software Version
	#define SW_MAJOR				1
	#define SW_MINOR				8
	#define SW_YEAR					2012
	#define SW_MONTH				2
	#define SW_DAY					15

	// Akku
	#define NICD					0
	#define NIMH					1
	#define BLEI					2
	#define LIION					3
	#define MAX_UNDERVOLTAGE		500 // mV
	#define MAX_UNDERVOLTAGE_CNT	60
	#define MAX_POWER				180000000 // uW
	#define MAX_CURRENT				5000 // mA

	// Ri Messen
	#define MIN_MEASURE_INTERVALL	1 // s
	#define MAX_MEASURE_INTERVALL	30 // s

	// Sprachen
	#define ANZ_SPRACHEN			2
	#define DEUTSCH					0
	#define ENGLISH					1

	// EEPROM
	#define EE_SPRACHE				0
	#define EE_AKKUTYP				1
	#define EE_ZELLENZAHL			2
	#define EE_ENTLADESTROM			3
	#define EE_SPEICHERN			7
	#define EE_LOGCOUNT				9
	#define EE_SCHLUSSSPANNUNG		10
	#define EE_KALIBRATION			14
	#define EE_MESSINTERVALL		18
	#define EE_ENTLADESTROM_RI		19

	#define EE_SPRACHE_MAX			1
	#define EE_AKKUTYP_MAX			3
	#define EE_ZELLENZAHL_MAX		25			
	#define EE_ENTLADESTROM_MAX		5000
	#define EE_SCHLUSSSPANNUNG_MAX	40000
	#define EE_SPEICHERN_MAX		900
	#define EE_LOGCOUNT_MAX			99
	#define EE_KALIBRATION_MAX		1.1
	#define EE_KALIBRATION_MIN		0.9
	#define EE_MESSINTERVALL_MAX	MAX_MEASURE_INTERVALL

	#define EE_SPRACHE_DEF			English
	#define EE_AKKUTYP_DEF			0
	#define EE_ZELLENZAHL_DEF		1			
	#define EE_ENTLADESTROM_DEF		10
	#define EE_SCHLUSSSPANNUNG_DEF	50
	#define EE_SPEICHERN_DEF		0
	#define EE_LOGCOUNT_DEF			1
	#define EE_KALIBRATION_DEF		1
	#define EE_MESSINTERVALL_DEF	5

	// Mens
	typedef enum {
		Aendern,
		Start,
	} FrameAnswer;

	typedef enum {
		DischargeSettings,
		RiSettings,
	} SettingsFrame;

	typedef enum {
		EntladenEnd,
		RiEnd,
	} EndMode;

	// Common
	#define	LOW						FALSE
	#define	HIGH					TRUE
	#define TRUE					1
	#define FALSE					0

	// Functions
	#define NOP() 					asm volatile ("nop" ::)

	// Hardware
	#define sbi(portn, bitn) 		((portn)|=(1<<(bitn)))
	#define cbi(portn, bitn) 		((portn)&=~(1<<(bitn)))

	// Kalibrierung
	#define SOLLADCHW				182 // 1024 / 2.56 * (5 / 11)
	
	//************
	//* Variabeln
	//************

	/**
	 * \struct AkkuStruct
	 * \brief Enthaelt informationen zum Akku und zur Entladung
	*/
	typedef struct {
		/** Akkutyp */
		unsigned char			Typ;
		/** Anzahl der Zellen im Akku */
		unsigned char			Zellenzahl;
		/** Entladestrom in mA*/
		unsigned int			Entladestrom;
		/** Entladeschlussspannung in mV */
		unsigned long			Entladespannung;
		/** Zeitintervall zum Abspeichern der Daten in s */
		unsigned int			Speicherintervall;
		/** Aktuelle Ladung mWs */
		volatile unsigned long	Energie;
		/** Aktuelle Kapazitaet mAs */
		volatile unsigned long	Kapazitaet;
		/** Aktuelle Spannung mV */
		volatile unsigned long	Spannung;
		/** Speichert die letzten 10 Spannungen; neuester 0 */
		volatile unsigned long	SpannungArray[10];
		/** Aktueller Strom mA */
		volatile unsigned int	Strom;
		/** Speichert die letzten 10 Stroeme; neuester 0 */
		volatile unsigned int	StromArray[10];
		/** Zaehlt die Anzahl Sekunden bei Unterspannung */
		volatile unsigned char	UnterspannungCounter;
		/** Zeit in s seit Entladevorgang */
		volatile unsigned long	Entladezeit;
	} AkkuStruct;
	
	/// \brief
	/// Struct um alle Einstellungen zur Innenwiderstand Messung abzuspeichern
	typedef struct {
		/// Maximaler Teststrom
		unsigned int			maxCurrent;
		/// Stromerhoehung pro Schritt [mA]
		unsigned int			currentStep;
		/// Strom Messpunkte (berechnet) [mA]
		unsigned int			currentPoints[21];
		/// Spannung bei entsprechendem Strom (currentPoints) [mV]
		unsigned long			voltagePoints[21];
		/// Effektiver Strom, der geflossen ist
		unsigned long			effectiveCurrentPoints[21];
		/// Messintervall in s
		unsigned char			messintervall;
		/// Mittelwert des Widerstands [Ohm]
		float					middleRi;
		/// Maximaler Innenwiderstand [Ohm]
		float					maxRi;
		/// wird per Interrupt erhoeht
		volatile unsigned int	msTick;
		/// Anzahl ms bis die Messung gestartet wir (letzte 25% des Messintervall)
		unsigned int 			startTime;
		/// Effektiver Intervall zwischen den Messungen (in den letzten 25%)
		unsigned int			intervall;
	} RiStruct;
	
	/**
	 * \struct TimeBase
	 * \brief Wird alle Sekunde aktualisiert
	*/
	typedef struct {
		/** Sekunden */
		volatile unsigned char	Sec;
		/** Minuten */
		volatile unsigned char	Min;
		/** Stunden */
		volatile unsigned char	Hour;
		/** Sekundenzaehler */
		volatile unsigned long	totSekunden;
		/** TRUE sobald Zeit geaendert hat  */
		volatile unsigned char	Changed;
	} TimeBase;

	/**
	 * \struct AuswahlStruct
	 * \brief Wird aktualisiert sobald der Encoder gedreht wird und Enable auf TRUE ist
	*/
	typedef struct {
		/** Maximalwert von Counter */
	    volatile unsigned int   MaxEintraege;
		/** Minimalwert von Counter */
	    volatile unsigned char	MinEintraege;
		/**  Counter */
	    volatile signed	  long	Count;
		/** Schrittaenderung bei normalem drehen */
		volatile unsigned char	Step;
		/** Schrittaenderung bei schnellem drehen */
		volatile unsigned int	BigStep;
		/** Wenn TRUE wird bei einem Counter ueberlauf am anderen Ende weitergezaehlt */
		volatile unsigned char	Roll;
		/** TRUE wenn Counter geaendert hat */
		volatile unsigned char	Changed;
		/** TRUE falls Aenderungen am Encoder ausgewertet werden sollen */
		volatile unsigned char	Enable;
	} AuswahlStruct;

	/**
	 * \struct TasterStruct
	 * \brief Wird aktualisiert sobald die OK-Taste gedrueckt wird
	*/
	typedef struct {
		/** TRUE falls gedrueckt */
		volatile unsigned char	Kurz;
		/** ms Counter zur Entprellung */
		volatile unsigned char	Counter;
	} TasterStruct;	

	extern TimeBase Zeit;
	extern AuswahlStruct Auswahl;
	extern TasterStruct TasteOK;
	extern AkkuStruct Akku;
	extern RiStruct Ri;

	/** Kalibrierungsfaktor */
	extern volatile float Kalibration;

	/** Temperatur des Kuehlers in (Grad Celsius * 10) <br>  Wird jede Sekunde aktualisiert */
	extern volatile unsigned int CoolerTemp;

	/** Dateiname der Logdatei */
	extern char Filename[];

	/** Zahlenanteil des Dateinames der Logdatei */
	extern volatile volatile unsigned char LogCount;

	/** Puffer um Strings zu kopieren */
	extern char buffer1[20];

	/** Puffer um Strings zu kopieren */
	extern char buffer2[20];

	/** Array um Daten zu sortieren */
	extern volatile unsigned int Array[10];
	
	
	//*************************
	//* Funktionsdeklarationen
	//*************************

	// Rechnet den Absolutwert von zwei vorzeichenlose Integer Zahlen aus
	unsigned int abs_uint(unsigned int value1, unsigned int value2);

	// Gibt das HighByte aus einem 16bit Datentyp zurck
	unsigned char GetHiByte(unsigned int Data);

	// Gibt das LowByte aus einem 16bit Datentyp zurck
	unsigned char GetLoByte(unsigned int Data);

	// Sortiert die Daten, die im Lokalen Array "Array[]" gespeichert sind
	void Sortieren(void);

#endif
