/*******
 glcd.c
*******/


//***********
//* Includes
//***********
#include "glcd.h"
#include "small.h"
#include "medium.h"


//************
//* Variabeln
//************

unsigned char column = 0;
unsigned char page = 0;

/**
* \brief
* Schreibt einen Buchstaben in kleiner Schrift aufs LCD
* \note
* Die Position muss zuerst mit folgenden Variabeln eingestellt werden <br>
* - column (0 bis 127) <br>
* - page (0 bis 7)
* \param c ASCII-Zeichen
* \version
* 1.1 Leerstellen nicht mehr schreiben
* 1.0 Erste Version
*/
void LCD_WriteChar_Small(char c)
{
	unsigned char i;
	unsigned char data;

	LCD_SetAddress(column, page);

	for(i=0; i<6; i++)
	{
		data = pgm_read_byte(&Small_table[((c - 0x20) * 6) + i]);
		if (data)
		{
			ks0108WriteData(data);
			column++;
		}
	}

	ks0108WriteData(0x00);
	column++;
}


/**
* \brief
* Schreibt einen String in kleiner Schrift aufs LCD
* \note
* Die Position muss zuerst mit folgenden Variabeln eingestellt werden <br>
* - column ( 0 bis 127) <br>
* - page (0 bis 7)
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \version
* 1.1 Leerstellen nicht mehr schreiben
* 1.0 Erste Version
*/
void LCD_WriteString_Small(char *data)
{
	while (*data) 
	{
		if (*data == ' ')
		{
			ks0108WriteData(0x00);
			ks0108WriteData(0x00);
			ks0108WriteData(0x00);
			column += 3;
		}
		else
		{
			LCD_WriteChar_Small(*data);
		}
    	data++;
	}
}


/**
* \brief
* Schreibt einen String in kleiner Schrift in dei Mitte des LCD <br>
* Der String wird zwischen die LowerBound und UpperBound ausgerichtet
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \param LowerBound Spalte des Linken Endes
* \param UpperBound Spalte des Rechten Endes
* \version
* 1.0 Erste Version
*/
void LCD_WriteString_Small_Center(char *data, unsigned char LowerBound, unsigned char UpperBound)
{
	unsigned char Length;

	// Lnge des String hohlen
	Length = GetLength_String_Small(data);

	// Startwert ausrechnen
	column = ((UpperBound - LowerBound) - Length) / 2;
	column += LowerBound;

	// String schreiben
	LCD_WriteString_Small(data);
}


/**
* \brief
* Schreibt einen String in kleiner Schrift an den rechten Rand des LCD
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \param RightBound Gibt die SpaltenNr des Rechten Ende des Textes an
* \version
* 1.0 Erste Version
*/
void LCD_WriteString_Small_Right(char *data, unsigned char RightBound)
{
	unsigned char Length;

	// Lnge des String hohlen
	Length = GetLength_String_Small(data);

	// Startwert ausrechnen
	column = RightBound - Length;

	// String schreiben
	LCD_WriteString_Small(data);
}


/**
* \brief
* Gibt die Laenge in Pixeln eines Strings zurueck
* \warning
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \return unsigned cahr Laenge in Byte
* \version
* 1.0 Erste Version
*/
unsigned char GetLength_String_Small(char *data)
{
	unsigned char Length = 0;

	while (*data) 
	{
		if (*data == ' ')
		{
			Length += 3;
		}
		else
		{
			Length += GetLength_Char_Small(*data);
		}
    	data++;
	}

	return Length;
}


/**
* \brief
* Gibt die Laenge in Pixeln eines Chars zurueck
* \warning
* \param c char
* \return unsigned cahr Laenge in Byte
* \version
* 1.0 Erste Version
*/
unsigned char GetLength_Char_Small(char c)
{
	unsigned char i;
	unsigned char data;
	unsigned char Length = 0;

	for(i=0; i<6; i++)
	{
		data = pgm_read_byte(&Small_table[((c - 0x20) * 6) + i]);
		if (data)
		{
			Length++;
		}
	}

	Length++;

	return Length;
}


/**
* \brief
* Schreibt eine Integer Zahl in kleiner Schrift aufs LCD
* \warning
* Die Position muss zuerst mit folgenden Variabeln eingestellt werden <br>
* - column ( 0 bis 127) <br>
* - page (0 bis 7) <br>
* Die Funktion kann mit negativen Zahlen umgehen, jedoch muss das Vorzeichen selber geschrieben werden
* \param data 16bit Datentyp
* \version
* 1.0 Erste Version
*/
void LCD_WriteInteger_Small(int data)
{
	char buffer[6];

	ultoa(data, buffer, 10);
	LCD_WriteString_Small(buffer);
}


/**
* \brief
* Schreibt einen Buchstaben in mittlerer Schrift aufs LCD
* \warning
* Die Position muss zuerst mit folgenden Variabeln eingestellt werden <br>
* - column ( 0 bis 127) <br>
* - page (0 bis 7)
* \param c ASCII-Zeichen
* \version
* 1.0 Erste Version
*/
void LCD_WriteChar_Medium(char c)
{
	unsigned char Line1[20];
	unsigned char Line2[20];
	unsigned long Pointer;
	unsigned char With;
	unsigned char i = 0;


	// Offset der char tabelle
	Pointer = pgm_read_word(&Medium_offset_table[c-32]);
	
	// Zeichenlnge
	With = pgm_read_byte(&Medium_char_table[Pointer]);

	// Zeichen auslesen
	Pointer++;
	for(i=0; i<With; i++)
	{
		Line1[i] = pgm_read_byte(&Medium_char_table[Pointer]);
		Pointer++;
		Line2[i] = pgm_read_byte(&Medium_char_table[Pointer]);
		Pointer++;
	}
	
	// Adresse der ersten Zeile setzen 
	LCD_SetAddress(column, page);
	
	// Daten der ersten Zeile aufs LCD schreiben
	for(i=0; i<With; i++)
	{
		ks0108WriteData(Line1[i]);
	}
	
	// Adresse der zweiten Zeile setzen
	LCD_SetAddress(column, (page + 1));


	// Daten der zweiten Zeile aufs LCD schreiben
	for(i=0; i<With; i++)
	{
		ks0108WriteData(Line2[i]);
	}
		
	column += With;
}


/**
* \brief
* Schreibt einen String in mittlerer Schrift aufs LCD
* \warning
* Die Position muss zuerst mit folgenden Variabeln eingestellt werden <br>
* - column ( 0 bis 127) <br>
* - page (0 bis 7)
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \version
* 1.0 Erste Version
*/
void LCD_WriteString_Medium(char *data)
{
  while (*data) {
    LCD_WriteChar_Medium(*data);
	LCD_WriteChar_Medium(LEERZEICHEN);
    data++;
  }
}


/**
* \brief
* Schreibt einen String in mittlerer Schrift in dei Mitte des LCD <br>
* Der String wird zwischen die LowerBound und UpperBound ausgerichtet
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \param LowerBound Spalte des Linken Endes
* \param UpperBound Spalte des Rechten Endes
* \version
* 1.0 Erste Version
*/
void LCD_WriteString_Medium_Center(char *data, unsigned char LowerBound, unsigned char UpperBound)
{
	unsigned char Length;

	// Lnge des String hohlen
	Length = GetLength_String_Medium(data);

	// Startwert ausrechnen
	column = ((UpperBound - LowerBound) - Length) / 2;
	column += LowerBound;

	// String schreiben
	LCD_WriteString_Medium(data);
}


/**
* \brief
* Gibt die Laenge in Pixeln eines Strings zurueck
* \warning
* \param *data Char-Array (muss ein NULL-Terminate haben)
* \return unsigned cahr Laenge in Byte
* \version
* 1.0 Erste Version
*/
unsigned char GetLength_String_Medium(char *data)
{
	unsigned long Pointer;
	unsigned char With;
	unsigned char Length = 0;

	while (*data)
	{
		// Offset der char tabelle
		Pointer = pgm_read_word(&Medium_offset_table[*data-32]);
	
		// Zeichenlnge
		With = pgm_read_byte(&Medium_char_table[Pointer]);

		// Lnge aufaddieren
    	Length += With;
		Length += LEN_LEERZEICHEN;
    	data++;
	}

	// Lnge zurckgeben
	return Length;
}


/**
* \brief
* Schreibt eine Integer Zahl in mittlerer Schrift aufs LCD
* \warning
* Die Position muss zuerst mit folgenden Variabeln eingestellt werden <br>
* - column ( 0 bis 127) <br>
* - page (0 bis 7) <br>
* Die Funktion kann mit negativen Zahlen umgehen, jedoch muss das Vorzeichen selber geschrieben werden
* \param data  16bit Datentyp
* \version
* 1.0 Erste Version
*/
void LCD_WriteInteger_Medium(int data)
{
	char buffer[6];

	ultoa(data, buffer, 10);
	LCD_WriteString_Medium(buffer);
}


/**
* \brief
* Schreibt eine Integer Zahl in mittlerer Schrift in die Mitte aufs LCD
* Die Funktion kann mit negativen Zahlen umgehen, jedoch muss das Vorzeichen selber geschrieben werden
* \param data  (Integer)
* \param LowerBound Untere Grenze
* \param UpperBound Obere Grenze
* \version
* 1.0 Erste Version
*/
void LCD_WriteInteger_Medium_Center(int data, unsigned char LowerBound, unsigned char UpperBound)
{
	char buffer[6];

	ultoa(data, buffer, 10);
	LCD_WriteString_Medium_Center(buffer, LowerBound, UpperBound);
}


/**
* \brief
* Schreibt eine Zahl, die 2 Ziffern hat. <br>
* Die restliche Ziffer wird mit einer vorangestellten 0 gefuellt.
* \param data Integer -99 bis 99
* \warning
* Die Funktion kann mit negativen Zahlen umgehen, jedoch muss das Vorzeichen selbst geschrieben werden
* \version
* 1.0 Erste Version
*/
void LCD_WriteZahl10_Medium(int data)
{
	if (data < 10)
	{
		LCD_WriteString_Medium("0");
	}

	LCD_WriteInteger_Medium(data);
}


/**
* \brief
* Schreibt eine Zahl, die 3 Ziffern hat. <br>
* Die restlichen Ziffern werden mit einer vorangestellten 0 gefuellt.
* \param data Integer -999 bis 999
* \warning
* Die Funktion kann mit negativen Zahlen umgehen, jedoch muss das Vorzeichen selbst geschrieben werden
* \version
* 1.0 Erste Version
*/
void LCD_WriteZahl100_Medium(int data)
{
	if (data < 10)
	{
		LCD_WriteString_Medium("0");
	}

	if (data < 100)
	{
		LCD_WriteString_Medium("0");
	}

	LCD_WriteInteger_Medium(data);
}


/**
* \brief
* Setzt die Adresse und rechnet die Page in Pixel um.
* \param column 0 bis 127
* \param page 0 bis 7
* \version
* 1.0 Erste Version
*/
void LCD_SetAddress(unsigned char column, unsigned char page)
{
	ks0108GotoXY(column, (page * 8));
}


//*************************************************************
/// \brief
/// Zeichent einen Fortschrittbalken auf das LCD
/// \note
/// Der Balken ist eine volle Zeile lang und 2 pages hoch (128 x 16)
/// \param
/// percentage Prozentzahl des Fortschrittes (0 bis 100)
/// \param
/// line Zeile (1 bis 7)
//*************************************************************
void writeBarGraph ( unsigned char percentage, unsigned char line ) {
	unsigned char oldPage, oldColumn;
	unsigned int width;
	char str[5];
	
	oldPage = page;
	oldColumn = column;
	
	// Flche lschen
	ks0108FillRect(0, line * 8 - 2, 127, 16, WHITE);

	// Text schreiben
	ultoa(percentage, str, 10);
	strcat(str, "%");
	column = 0;
	page = line;
	LCD_WriteString_Medium_Center(str, 0, 127);

	// Rahmen zeichnen
	ks0108FillRect(0, 	line * 8 - 2, 		127, 0, BLACK);
	ks0108FillRect(0, 	line * 8 - 2 + 16, 	127, 0, BLACK);
	ks0108FillRect(0, 	line * 8 - 2, 		0, 	16, BLACK);
	ks0108FillRect(127, line * 8 - 2, 		0, 	16, BLACK);
	
	// Flche Zeichen
	width = (125 * percentage) / 100;
	if (width > 0) {
		ks0108InvertRect(1, line * 8 - 1, (unsigned char)width, 14);
	}
	
	page = oldPage;
	column = oldColumn;
}
