/******
 ADC.c
******/


//***********
//* Includes
//***********
#include "adc.h"


/**
* \brief
* Liest einen Analogeingang ein. Die Referenz und der Kanal und  muss vorher gewaehlt werden
* \note Die Referenz muss vor dem Kanal gesetzt werden
* \return 10bit AD-Wert
* \version
* 1.0 Erste Version
*/
unsigned int ADC_Read(void)
{
	int Resultat;
	int i;

	// Dummy Read
	ADCSRA |= (1<<ADSC);
	while (ADCSRA & (1<<ADSC)) 
	{
		;
	}

	// "single conversion" einstellen
    ADCSRA |= (1<<ADSC);							
	Resultat = 0;

	// Eigentliche Messung
	for (i=0; i<4; i++)
	{
		// auf Abschluss der Konvertierung warten
		while (ADCSRA & (1<<ADSC)) {;}

		// Messung aufaddieren
		Resultat += ADCW;
	}								 

 	// Resultat durch vier teilen
	Resultat /= 4;
 
  return Resultat;
}


/**
* \brief
* Setzt die Kanalnummer
* \param mux Kanal 0 bis 7
* \note Vorher muss die Referenz festgelegt werden
* \version
* 1.0 Erste Version
*/
void Set_Channel(unsigned char mux)
{
	unsigned char byte;

	byte = ADMUX;
	byte = (byte & 0xF0);
	byte = (byte | mux);

	ADMUX = byte;
}


/**
* \brief
* Waehlt die interne Referenzspannung aus
* \version
* 1.0 Erste Version
*/
void Set_ADRefInternal(void)
{
	ADMUX |= ((1<<REFS1) | (1<<REFS0));
}


/**
* \brief
* Waehlt die Betriebsspannung AVCC als Referenzspannung aus
* \version
* 1.0 Erste Version
*/
void Set_ADRefVCC(void)
{
	ADMUX &= ~(1<<REFS1);
	ADMUX |= (1<<REFS0);
}


/**
* \brief
* Liest die Temperatur ein
* \return Temperatur in (Grad Celsius * 10)
* \version
* 1.0 Erste Version
*/
unsigned int Read_Temp(void)
{   
	unsigned int ADWert;
	float Voltage; // (mV)

	/*-------------------------------
	 Spannung = ReferenzSpannung /
	 Auflsung * ADC /

	 U = 5000mV / 1024 * ADWert 
	 U = 4.88 * ADWert
	-------------------------------*/

	// AVCC als Referenz
	Set_ADRefVCC();
	_delay_ms(1);

	// Temperatur messen
	Set_Channel(TEMP_PINNR);	
	ADWert = ADC_Read();
	
	// Temperatur ausrechnen
	Voltage = 4.88 * ADWert;
	Voltage -= 2730;
	
	/*-------------------------------
	 Der LM335 gibt pro C 10mV
	 Das heisst, dass das gleich unser
	 ergebnis ist. 253mV = 25.3C
	-------------------------------*/

	// Spannung als Integer zurckgeben
	return (unsigned int) Voltage; 
}


/**
* \brief
* Liest die Spannung ein
* \return Spannung in mV
* \version
* 1.1 Auf 2 Referenzen verteilt
* 1.0 Erste Version
*/
unsigned long Read_Voltage(void)
{   
	unsigned int ADWert;
	float Voltage; // (mV)

	/*-------------------------------
	 Spannung = ReferenzSpannung /
	 Auflsung * ADC * Spannungsteiler

	 Spannungsteiler = 11
	-------------------------------*/
	
	// Interne Referenz
	Set_ADRefInternal();
	_delay_ms(1);

	// UBatt messen
	Set_Channel(VOLTAGE_PINNR);	
	ADWert = ADC_Read();

	// Kalibrierungsfaktor
	ADWert *= Kalibration;

	if (ADWert < 950)
	{
		/*-------------------------------
		 U = 2560mV / 1024 * ADWert * 11
		 U = 27.5 * ADWert;
		-------------------------------*/
		Voltage = 27.5 * ADWert;
	}
	else
	{
		/*-------------------------------
		 U = 5000mV / 1024 * ADWert * 11
		 U = 53.711 * ADWert;
		-------------------------------*/

		// AVCC als Referenz
		Set_ADRefVCC();
		_delay_ms(1);

		// UBatt messen
		Set_Channel(VOLTAGE_PINNR);
		ADWert = ADC_Read();
		Voltage = 53.711 * ADWert;
	}
	
	// Spannung als Integer zurckgeben
	return (unsigned long) Voltage; 
}


/**
* \brief
* Liest den Strom ein
* \return Spannung in mV
* \version
* 1.0 Erste Version
*/
unsigned int Read_Current(void)
{   
	unsigned int ADWert;
	float Current; // (mA)

	/*-------------------------------
	 I = Urmess / Rmess
	 Urmess = Ref / 1024 * ADWert * Spannungsteiler

	 Spannungsteiler = 2
	-------------------------------*/

	// Interne Referenz
	Set_ADRefInternal();
	_delay_ms(1);

	// IBatt messen
	Set_Channel(CURRENT_PINNR);	
	ADWert = ADC_Read();

	if (ADWert <950)
	{
		/*-------------------------------
		 Urmess = 2560mV / 1024 * ADWert * 2
		 Urmess = 5 * ADWert

		 I = 2.777 * ADWert
		-------------------------------*/
		Current = 2.777 * ADWert;
	}
	else
	{
		// AVCC als Referenz
		Set_ADRefVCC();
		_delay_ms(1);

		// IBatt messen
		Set_Channel(CURRENT_PINNR);	
		ADWert = ADC_Read();

		/*-------------------------------
		 Urmess = 5000mV / 1024 * ADWert * 2
		 Urmess = 9.765 * ADWert

		 I = 5.425 * ADWert
		-------------------------------*/
		Current = 5.425 * ADWert;
	}	

	// Strom als Integer zurckgeben
	return (unsigned int) Current;
}



