/********
eeprom.c
*********/

//***********
//* Includes
//***********
#include "eeprom.h"


/**
* \brief
* Liest die Daten aus dem EEPROM und speichert sie ins SRAM
* \version
* Version 2.0
*/
void EEPROM_To_SRAM(void)
{
	// Interrupts deaktivieren
	cli();

	// Sprache
	SetLanguage(readByteFromEeprom(EE_SPRACHE, 0, EE_SPRACHE_MAX, EE_SPRACHE_DEF));

	// Akkutyp
	Akku.Typ = readByteFromEeprom(EE_AKKUTYP, 0, EE_AKKUTYP_MAX, EE_AKKUTYP_DEF);

	// Zellenzahl
	Akku.Zellenzahl = readByteFromEeprom(EE_ZELLENZAHL, 1, EE_ZELLENZAHL_MAX, EE_ZELLENZAHL_DEF);

	// Entladestrom
	Akku.Entladestrom = readIntegerFromEeprom(EE_ENTLADESTROM, 0, EE_ENTLADESTROM_MAX, EE_ENTLADESTROM_DEF);

	// Schlussspannung
	Akku.Entladespannung = readLongFromEeprom(EE_SCHLUSSSPANNUNG, 0, EE_SCHLUSSSPANNUNG_MAX, 0);
	if (Akku.Entladespannung == 0) {
		SchlussspannungAusrechnen();
	}	
	
	// Speicherintervall
	Akku.Speicherintervall = readIntegerFromEeprom(EE_SPEICHERN, 0, EE_SPEICHERN_MAX, EE_SPEICHERN_DEF);

	// Logdatei Counter
	LogCount = readByteFromEeprom(EE_LOGCOUNT, 0, EE_LOGCOUNT_MAX, EE_LOGCOUNT_DEF);

	// Kalibrierung
	Kalibration = readFloatFromEeprom(EE_KALIBRATION, EE_KALIBRATION_MIN, EE_KALIBRATION_MAX, EE_KALIBRATION_DEF);
	
	// Messintervall
	Ri.messintervall = readByteFromEeprom(EE_MESSINTERVALL, 1, EE_MESSINTERVALL_MAX, EE_MESSINTERVALL_DEF);
	
	// max. Entladestrom Ri
	Ri.maxCurrent = readIntegerFromEeprom(EE_ENTLADESTROM_RI, 0, EE_ENTLADESTROM_MAX, EE_ENTLADESTROM_DEF);

	// Interrupts aktivieren
	sei();
}


/**
* \brief
* Speichert die Daten vom SRAM ins EEPROM
* \version
* Version 2.0
*/
void SRAM_To_EEPROM(void)
{
	// Interrupts deaktivieren
	cli();

	// Sprache
	writeByte2eeprom(EE_SPRACHE, GetLanguage());

	// Akkutyp
	writeByte2eeprom(EE_AKKUTYP, Akku.Typ);

	// Zellenzahl
	writeByte2eeprom(EE_ZELLENZAHL, Akku.Zellenzahl);

	// Entladestrom
	writeInteger2eeprom(EE_ENTLADESTROM, Akku.Entladestrom);

	// Schlussspannung
	writeLong2eeprom(EE_SCHLUSSSPANNUNG, Akku.Entladespannung);

	// Speicherintervall
	writeInteger2eeprom(EE_SPEICHERN, Akku.Speicherintervall);

	// Logdatei Counter
	writeByte2eeprom(EE_LOGCOUNT, LogCount);

	// Kalibrierung
	writeFloat2eeprom(EE_KALIBRATION, Kalibration);
	
	// Messintervall Ri
	writeByte2eeprom(EE_MESSINTERVALL, Ri.messintervall);
	
	// Entladestrom Ri
	writeInteger2eeprom(EE_ENTLADESTROM_RI, Ri.maxCurrent);

	// Interrupts aktivieren
	sei();
}

void writeByte2eeprom ( unsigned int adress, unsigned char val ) {
	EEPROM_write(adress, val);
}

void writeInteger2eeprom ( unsigned int adress, unsigned int val ) {
	EEPROM_write(adress, (unsigned char) (val));
	EEPROM_write((adress + 1), (unsigned char) (val >> 8));
}

void writeLong2eeprom ( unsigned int adress, unsigned long val ) {
	EEPROM_write(adress, (unsigned char) (val));
	EEPROM_write((adress + 1), (unsigned char) (val >> 8));
	EEPROM_write((adress + 2), (unsigned char) (val >> 16));
	EEPROM_write((adress + 3), (unsigned char) (val >> 24));
}

void writeFloat2eeprom ( unsigned int adress, float val ) {
	float2long tmp;
	tmp.f_value = val;
	writeLong2eeprom(adress, tmp.l_value);
}

unsigned char readByteFromEeprom ( unsigned int adress, unsigned char lBound, unsigned char uBound, unsigned char def ) {
	unsigned char val;
	val = EEPROM_read(adress);
	
	if ((val > uBound) || (val < lBound)) {
		val = def;
	}
	return val;
}

unsigned int readIntegerFromEeprom ( unsigned int adress, unsigned int lBound, unsigned int uBound, unsigned int def ) {
	unsigned int val;
	val = EEPROM_read(adress + 1);
	val <<= 8;
	val += EEPROM_read(adress);
	
	if ((val > uBound) || (val < lBound)) {
		val = def;
	}
	return val;
}

unsigned long readLongFromEeprom ( unsigned int adress, unsigned long lBound, unsigned long uBound, unsigned long def ) {
	unsigned long val;
	val = EEPROM_read(adress + 3);
	val <<= 8;
	val |= EEPROM_read(adress + 2);
	val <<= 8;
	val |= EEPROM_read(adress + 1);
	val <<= 8;
	val |= EEPROM_read(adress);
	
	if ((val > uBound) || (val < lBound)) {
		val = def;
	}
	return val;
}

float readFloatFromEeprom ( unsigned int adress, float lBound, float uBound, float def ) {
	float2long tmp;
	tmp.l_value = readLongFromEeprom(adress, 0, 0xffffffff, 0);
	
	if ((tmp.f_value > uBound) || (tmp.f_value < lBound)) {
		tmp.f_value = def;
	}
	return tmp.f_value;
}

/**
* \brief
* Liest ein Byte aus dem internen EEPROM
* \param
* uiAddress Adresse im EEPROM 0 - 511
* \return
* Byte das ausgelesen wurde
* \version
* Version 1.0
*/
unsigned char EEPROM_read(unsigned int uiAddress)
{
	/* Wait for completion of previous write */
	while(EECR & (1<<EEWE)) {;}

	/* Set up address register */
	EEAR = uiAddress;

	/* Start eeprom read by writing EERE */
	EECR |= (1<<EERE);

	/* Return data from data register */
	return EEDR;
}


/**
* \brief
* Schreibt ein Byte in das internen EEPROM
* \param uiAddress Adresse im EEPROM 0 - 511
* \param ucData DAtenbyte
* \version
* Version 1.0
*/
void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
	/* Wait for completion of previous write */
	while(EECR & (1<<EEWE)) {;}

	/* Set up address and data registers */
	EEAR = uiAddress;
	EEDR = ucData;

	/* Write logical one to EEMWE */
	EECR |= (1<<EEMWE);

	/* Start eeprom write by setting EEWE */
	EECR |= (1<<EEWE);
}
