/*********
 ks0108.c
**********/


//***********
//* Includes
//***********
#include "ks0108.h"


//***********
//* Variabeln
//***********
lcdCoord			ks0108Coord;
unsigned char		ks0108Inverted=0;


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Springt an die Adresse X, Y
* \param x (0 - 127)
* \param y (0 - 63)
*/
void ks0108GotoXY(unsigned char x, unsigned char y)
{
	unsigned char chip = CHIP1, cmd;
	
	if(x > 127) x = 0;								// ensure that coordinates are legal
	if(y > 63)  y = 0;
	
	ks0108Coord.x = x;								// save new coordinates
	ks0108Coord.y = y;
	ks0108Coord.page = y/8;
	
	if(x >= 64) {									// select the right chip
		x -= 64;
		chip = CHIP2;
	}
	cmd = LCD_SET_ADD | x;
	ks0108WriteCommand(cmd, chip);					// set x address on active chip
	
	cmd = LCD_SET_PAGE | ks0108Coord.page;			// set y address on both chips
	ks0108WriteCommand(cmd, CHIP1);
	ks0108WriteCommand(cmd, CHIP2);
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Initialisiert das LCD
* \param invert (Wenn TRUE wird das LCD invertiert)
*/
void ks0108Init(unsigned char invert)
{
	ks0108Coord.x = 0;
	ks0108Coord.y = 0;
	ks0108Coord.page = 0;
	
	// Initialisiert die Hardware
	ks0108InitHW();
	
	// Reset lsen
	ks0108Reset(FALSE);

	// Clear Lcd
	ks0108ClearScreen();
	
	ks0108Inverted = invert;
	
	ks0108WriteCommand(LCD_ON, CHIP1);				// power on
	ks0108WriteCommand(LCD_ON, CHIP2);
	
	ks0108WriteCommand(LCD_DISP_START, CHIP1);		// display start line = 0
	ks0108WriteCommand(LCD_DISP_START, CHIP2);
	ks0108ClearScreen();							// display clear
	ks0108GotoXY(0,0);
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Setzt den Reset Pin
* \param
* resetState (0 = LOW; 1 = HIGH)
*/
void ks0108Reset(unsigned char resetState)
{
	if(resetState)
	{
		LCD_RESET_LOW();
	}
	else
	{
		LCD_RESET_HIGH();
	}
}

/**
* \author
* Fabian Maximilian Thiele
* \brief
* Aktiviert das LCD
*/
void ks0108Enable(void)
{
	volatile unsigned char i;
	
	LCD_EN_HIGH();									// EN high level width: min. 450ns
	asm volatile("nop\n\t"
				 "nop\n\t"
				 "nop\n\t"
				 ::);
	LCD_EN_LOW();
	for(i=0; i<8; i++) {;}								// a little delay loop (faster than reading the busy flag)
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Liest daten aus dem LCD
* \param first (unknown)
*/
unsigned char ks0108DoReadData(unsigned char first)
{
	unsigned char data;
	volatile unsigned char i;
							
	LCD_Write_Out(0x00);
	LCD_Dir_In();									// data port is input
	
	if(ks0108Coord.x < 64) {
		LCD_CS2_LOW();								// deselect chip 2
		LCD_CS1_HIGH();								// select chip 1
	} else if(ks0108Coord.x >= 64) {
		LCD_CS1_LOW();								// deselect chip 1
		LCD_CS2_HIGH();								// select chip 2
	}
	if(ks0108Coord.x == 64 && first) {				// chip2 X-address = 0
		ks0108WriteCommand(LCD_SET_ADD, CHIP2); 	// wuff wuff
	}
	
	LCD_DI_HIGH();									// D/I = 1
	LCD_RW_HIGH();									// R/W = 1
	
	//LCD_CMD_PORT |= 0x01 << EN;						// EN high level width: min. 450ns
	LCD_EN_HIGH();									// EN high level width: min. 450ns
	asm volatile("nop\n\t"
				 "nop\n\t"
				 "nop\n\t"
				 ::);
	
	data = LCD_Read_In();								// read Data			 
	
	LCD_EN_LOW();
	for(i=0; i<8; i++);								// a little delay loop (faster than reading the busy flag)
	
	LCD_Dir_Out();
	
	ks0108GotoXY(ks0108Coord.x, ks0108Coord.y);
	
	if(ks0108Inverted)
		data = ~data;
	return data;
}


/**
* \author
* Fabian Maximilian Thiele
 * \brief
 * Liest Daten aus dem LCD inkl. Dummy-Read
*/
unsigned char ks0108ReadData(void)
{
	ks0108DoReadData(1);							// dummy read
	return ks0108DoReadData(0);						// "real" read
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Schreibt einen Befehl zum LCD
* \param cmd (Command)
* \param chip (CHIP1 oder CHIP2)
*/
void ks0108WriteCommand(unsigned char cmd, unsigned char chip)
{
	if(chip == CHIP1) {
		LCD_CS2_LOW();								// deselect chip 2
		LCD_CS1_HIGH();								// select chip 1
	} else if(chip == CHIP2) {
		LCD_CS1_LOW();								// deselect chip 1
		LCD_CS2_HIGH();								// select chip 2
	}
	
	LCD_DI_LOW();									// D/I = 0
	LCD_RW_LOW();									// R/W = 0
	LCD_Dir_Out();									// data port is output
	LCD_Write_Out(cmd);								// write command
	ks0108Enable();									// enable
	LCD_Write_Out(0x00);
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Schreibt Daten zum LCD
*/
void ks0108WriteData(unsigned char data)
{
	unsigned char displayData, yOffset, cmdPort;

	if(ks0108Coord.x >= 128)
		return;

	if(ks0108Coord.x < 64) {
		LCD_CS2_LOW();								// deselect chip 2
		LCD_CS1_HIGH();								// select chip 1
	} else if(ks0108Coord.x >= 64) {
		LCD_CS1_LOW();								// deselect chip 1
		LCD_CS2_HIGH();								// select chip 2
	}
	if(ks0108Coord.x == 64)							// chip2 X-address = 0
		ks0108WriteCommand(LCD_SET_ADD, CHIP2);
	
	LCD_DI_HIGH();									// D/I = 1
	LCD_RW_LOW();									// R/W = 0
	LCD_Dir_Out();									// data port is output
	
	
	yOffset = ks0108Coord.y%8;
	if(yOffset != 0) {
		// first page
		cmdPort = LCD_Save_Cmd();					// save command port
		displayData = ks0108ReadData();
		
		LCD_Restore_Cmd(cmdPort);					// restore command port
		LCD_Dir_Out();								// data port is output
		
		displayData |= data << yOffset;
		if(ks0108Inverted)
			displayData = ~displayData;
		LCD_Write_Out(displayData);					// write data
		ks0108Enable();								// enable
		
		// second page
		ks0108GotoXY(ks0108Coord.x, ks0108Coord.y+8);
		
		displayData = ks0108ReadData();
		
		LCD_Restore_Cmd(cmdPort);					// restore command port
		LCD_Dir_Out();								// data port is output
		
		displayData |= data >> (8-yOffset);
		if(ks0108Inverted)
			displayData = ~displayData;
		LCD_Write_Out(displayData);					// write data
		ks0108Enable();								// enable
		
		ks0108GotoXY(ks0108Coord.x+1, ks0108Coord.y-8);
	} else {
		if(ks0108Inverted)
			data = ~data;
		LCD_Write_Out(data);						// write data
		ks0108Enable();								// enable
		ks0108Coord.x++;
	}
	LCD_Write_Out(0x00);
}


/**
* \author
* Philipp Klin
* \brief
* LCD Datenrichtung Out
*/
void LCD_Dir_Out(void)
{
	LCD_DATAH_PORT &= 0x0f;
	LCD_DATAL_PORT &= 0xf0;

	LCD_DATAH_DDR |= 0xf0;
	LCD_DATAL_DDR |= 0x0f;
}


/**
* \author
* Philipp Klin
* \brief
* LCD Datenrichtung In
*/
void LCD_Dir_In(void)
{
	LCD_DATAH_PORT &= 0x0f;
	LCD_DATAL_PORT &= 0xf0;

	LCD_DATAH_DDR &= 0x0f;
	LCD_DATAL_DDR &= 0xf0;
}


/**
* \author
* Philipp Klin
* \brief
* Liest die Daten vom LCD ein
* \return Datenbyte
*/
unsigned char LCD_Read_In(void)
{
	unsigned char data;

	data = (LCD_DATAH_PIN & 0xf0);
	data |= (LCD_DATAL_PIN & 0x0f);

	return data;
}


/**
* \author
* Philipp Klin
* \brief
* Sendet die Daten ans LCD
* \param data (Datenbyte)
*/
void LCD_Write_Out(unsigned char data)
{
	LCD_DATAH_PORT |= (data & 0xf0);
	LCD_DATAL_PORT |= (data & 0x0f);
}


/**
* \author
* Philipp Klin
* \brief
* Speichert die Zustaende der Steuerleitungen
* \return Commandlines
*/
unsigned char LCD_Save_Cmd(void)
{
	unsigned char data;

	if (LCD_DI_PORT & (1 << LCD_DI_PINNR))
	{
		data = 0b00000001;
	}

	if (LCD_RW_PORT & (1 << LCD_RW_PINNR))
	{
		data |= 0b00000010;
	}

	if (LCD_EN_PORT & (1 << LCD_EN_PINNR))
	{
		data |= 0b00000100;
	}

	if (LCD_CS1_PORT & (1 << LCD_CS1_PINNR))
	{
		data |= 0b00001000;
	}

	if (LCD_CS2_PORT & (1 << LCD_CS2_PINNR))
	{
		data |= 0b00010000;
	}

	if (LCD_RESET_PORT & (1 << LCD_RESET_PINNR))
	{
		data |= 0b00100000;
	}

	return data;
}


/**
* \author
* Philipp Klin
* \brief
* Stellt die Zustaende der Steuerleitungen wieder her
* \param cmd (Die zuvor in ein Byte gespeicherten Commandlines)
*/
void LCD_Restore_Cmd(unsigned char cmd)
{
	if (cmd & 0b00000001)
	{
		LCD_DI_HIGH();
	}
	else
	{
		LCD_DI_LOW();
	}

	if (cmd & 0b00000010)
	{
		LCD_RW_HIGH();
	}
	else
	{
		LCD_RW_LOW();
	}

	if (cmd & 0b00000100)
	{
		LCD_EN_HIGH();
	}
	else
	{
		LCD_EN_LOW();
	}

	if (cmd & 0b00001000)
	{
		LCD_CS1_HIGH();
	}
	else
	{
		LCD_CS1_LOW();
	}

	if (cmd & 0b00010000)
	{
		LCD_CS1_HIGH();
	}
	else
	{
		LCD_CS2_LOW();
	}

	if (cmd & 0b00100000)
	{
		LCD_RESET_HIGH();
	}
	else
	{
		LCD_RESET_LOW();
	}
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Fuellt ein Rechteck aus
* \param x (X-Koordinate)
* \param y (Y-Koordinate)
* \param width (Breite)
* \param height (Hoehe)
* \param color (WHITE oder BLACK)
*/
void ks0108FillRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char color)
{
	unsigned char mask, pageOffset, h, i, data;
	height++;
	
	pageOffset = y%8;
	y -= pageOffset;
	mask = 0xFF;
	if(height < 8-pageOffset) {
		mask >>= (8-height);
		h = height;
	} else {
		h = 8-pageOffset;
	}
	mask <<= pageOffset;
	
	ks0108GotoXY(x, y);
	for(i=0; i<=width; i++) {
		data = ks0108ReadData();
		
		if(color == BLACK) {
			data |= mask;
		} else {
			data &= ~mask;
		}

		ks0108WriteData(data);
	}
	
	while(h+8 <= height) {
		h += 8;
		y += 8;
		ks0108GotoXY(x, y);
		
		for(i=0; i<=width; i++) {
			ks0108WriteData(color);
		}
	}
	
	if(h < height) {
		mask = ~(0xFF << (height-h));
		ks0108GotoXY(x, y+8);
		
		for(i=0; i<=width; i++) {
			data = ks0108ReadData();
		
			if(color == BLACK) {
				data |= mask;
			} else {
				data &= ~mask;
			}
	
			ks0108WriteData(data);
		}
	}
}


/**
* \author
* Fabian Maximilian Thiele
* \brief
* Invertiert ein Rechteck
* \param x (X-Koordinate)
* \param y (Y-Koordinate)
* \param width (Laenge)
* \param height (Hoehe)
*/
void ks0108InvertRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height)
{
	unsigned char mask, pageOffset, h, i, data, tmpData;

	height++;
	
	pageOffset = y%8;
	y -= pageOffset;
	mask = 0xFF;
	if(height < 8-pageOffset) {
		mask >>= (8-height);
		h = height;
	} else {
		h = 8-pageOffset;
	}
	mask <<= pageOffset;
	
	ks0108GotoXY(x, y);
	for(i=0; i<=width; i++) {
		data = ks0108ReadData();
		tmpData = ~data;
		data = (tmpData & mask) | (data & ~mask);
		ks0108WriteData(data);
	}
	
	while(h+8 <= height) {
		h += 8;
		y += 8;
		ks0108GotoXY(x, y);
		
		for(i=0; i<=width; i++) {
			data = ks0108ReadData();
			ks0108WriteData(~data);
		}
	}
	
	if(h < height) {
		mask = ~(0xFF << (height-h));
		ks0108GotoXY(x, y+8);
		
		for(i=0; i<=width; i++) {
			data = ks0108ReadData();
			tmpData = ~data;
			data = (tmpData & mask) | (data & ~mask);
			ks0108WriteData(data);
		}
	}
}

